from revChatGPT.V3 import ChatbotCLI

def initialize(settings)-> ChatbotCLI:
    chatbot = ChatbotCLI(
        api_key=settings["api_key"],
        system_prompt=settings["base_prompt"],
        proxy=settings["proxy"],
        temperature=settings["temperature"],
        top_p=settings["top_p"],
        reply_count=settings["reply_count"],
        engine=settings["model"],

    )
    return chatbot




import requests
import json
import logging
import timeit



class InternetModule:
    def __init__(self, bot) -> None:
        self.bot = bot

    def get_prompt_wrapper_for_query(self, prompt):
        prompt_wrapper = f'This is a prompt from a user to a chatbot: "{prompt}". Respond with "none" if it is directed at the chatbot or cannot be answered by an internet search. Otherwise, respond with a possible search query to a search engine. Do not write any additional text. Make it as minimal as possible'
        return prompt_wrapper

    def ask_using_internet(self, prompt, role="user", convo_id="search", temperature=0, internet_search_depth=3):
        output, query_time = self._time_function(
            lambda: self._ask_using_internet(prompt, role, convo_id, temperature, internet_search_depth),
        )
        output["query_time"] = query_time
        return output

    def _ask_using_internet(self, prompt, role="user", convo_id="search", temperature=0, internet_search_depth=3):
        output = {}  # Initialize an empty dictionary to store output

        bot = self.bot

        prompt_wrapped = self.get_prompt_wrapper_for_query(prompt)
        query = bot.ask(prompt_wrapped, convo_id="search", temperature=1)
        output["query"] = query
        search_results = '{"results": "No search results"}'
        if query != "none":
            def get_search_results():
                nonlocal search_results
                resp = requests.post(
                    url="https://ddg-api.herokuapp.com/search",
                    json={"query": query, "limit": internet_search_depth},
                    timeout=10,
                )
                resp.encoding = "utf-8" if resp.encoding is None else resp.encoding
                search_results = resp.text

            search_time = self._time_function(get_search_results)

            try:
                logging.info(search_results)  # Print search_results to see the data
                output["search_results"] = json.loads(search_results)
                output["search_time"] = search_time
                bot.add_to_conversation(
                    f"Search results:{search_results}",
                    "system",
                    convo_id="default",
                )
                output["response"] = bot.ask(prompt, "user", convo_id="default")  # Append output to dictionary
            except json.decoder.JSONDecodeError as e:
                logging.error("Error parsing JSON data:", e)
            except StopIteration as e:
                logging.error("StopIteration:", e)
        else:
            output["response"] = self.ask_default(prompt, role, convo_id, temperature)

        return output

    def ask_default(self, prompt, role="user", convo_id="default", temperature=0):
        response, default_time = self._time_function(
            lambda: self.bot.ask(prompt, role=role, convo_id=convo_id, temperature=temperature)
        )
        return {"response": response, "default_time": default_time}

    def _time_function(self, func):
        start_time = timeit.default_timer()
        result = func()
        end_time = timeit.default_timer()
        elapsed_time = end_time - start_time
        return result, elapsed_time


from chrono_chatgpt.internet_module import InternetModule
import settings.setting_loader as setting_loader
import chrono_chatgpt.bot_initializer as bot_initializer
import logging


class ChatBotCore:
    def __init__(self):
        bot_settings = setting_loader.load()
        self.bot = bot_initializer.initialize(bot_settings)
        self.chatbot = self.bot
        #self.mode_path = "C:\\Users\\Terre\\Development\\_Experiments\\ChatGPT\\src\\revChatGPT\\config\\modes.json"
        self.mode_path = "settings\\modes.json"
        self.internet_module = InternetModule(self.bot)
        self.prompt_wrapper = lambda s: s

        self.internet_enabled = False
        self.is_internet_config_loaded = False

    def load_process_file_config(self):
        self.bot.load(self.mode_path, "conversation")


    def enable_default(self):
        self.prompt_wrapper = lambda prompt: f'{prompt}'

    def enable_internet(self):
        self.prompt_wrapper = lambda prompt: f'This is a prompt from a user to a chatbot: "{prompt}". Respond with "none" if it is directed at the chatbot or cannot be answered by an internet search. Otherwise, respond with a possible search query to a search engine. Do not write any additional text. Make it as minimal as possible'
        self.internet_enabled = True

    def enable_process_file(self):
        self.prompt_wrapper = lambda prompt: f"This is a file containing the filename of the file and file content from a user to a chatbot: {prompt}, do not talk about yourself, do not respond conversationally, and do not explain. just reply ok"

    def load_config(self):
        if not self.is_internet_config_loaded:
            self.bot.load(self.mode_path, "conversation")
            self.internet_enabled = True
            logging.info("internet config loaded!")
            self.is_internet_config_loaded = True


    def get_token_count(self, convo_id: str = "default") -> int:
        return self.chatbot.get_token_count(convo_id)

    def get_max_tokens(self, convo_id: str) -> int:
        return self.chatbot.get_max_tokens(convo_id)

    def ask_core(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        temperature = 0,
        **kwargs,
    ) -> str:
        output="None"
        prompt_wrapper = self.prompt_wrapper
        if prompt_wrapper is not None:
            prompt = prompt_wrapper(prompt)

        if self.internet_enabled:
            output = self.internet_module.ask_using_internet(prompt, role, convo_id, temperature=temperature)
        else:
            output = self.internet_module.ask_default(prompt=prompt, role=role, convo_id=convo_id)
        return output

    def ask(self, prompt: str, role: str = "user", convo_id: str = "default", temperature=0, response_only = True, **kwargs) -> str:
        max_depth = 10
        for _ in range(max_depth):
            result = self.ask_core(prompt, role, convo_id, temperature)
            if "response" in result and response_only:
                return result["response"]
            else:
                return result
        return None

    def ask_full(self, prompt: str, role: str = "user", convo_id: str = "default", temperature=0, response_only = True, **kwargs) -> str:
        return self.ask(prompt, role, convo_id, temperature, response_only=False)

    def ask_stream(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        **kwargs,
    ) -> str:
        return self.chatbot.ask_stream(prompt, role, convo_id, **kwargs)

    def rollback(self, n: int = 1, convo_id: str = "default") -> None:
        self.chatbot.rollback(n, convo_id)

    def reset(self, convo_id: str = "default", system_prompt: str = None) -> None:
        self.chatbot.reset(convo_id, system_prompt)

    def save(self, file: str, *keys: str) -> None:
        self.chatbot.save(file, *keys)

    def load(self, file: str, *keys: str) -> None:
        self.chatbot.load(file, *keys)



import setuptools
from setuptools import version

with open("README.md", 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='chrono_chatgpt',
    version='0.2',
    author="ChronoWalker",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/liaoyuzh/chrono-chatgpt",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3"
    ]
)